#ifndef _h_POINT_
#define _h_POINT_

#ifndef _h_TYPES_
# include <cv/types.h>
#endif


struct point;
struct rect;

typedef point Point, TPoint;
typedef rect Rect, TRect;
typedef const rect &CRect;


/*---------------------------------------------------------*/
struct point{
    int x, y;

    point()		:x(0), y(0) {}
    point(int xy)	:x(xy), y(xy) {}
    point(int x, int y)	:x(x), y(y) {}

    point &operator += (point p)	{ x += p.x, y += p.y; return *this; }
    point &operator -= (point p)	{ x -= p.x, y -= p.y; return *this; }
    point operator + (point p) const	{ p.x += x, p.y += y; return p; }
    point operator - (point p) const	{ p.x = x - p.x, p.y = y - p.y; return p; }
    point operator * (int v) const	{ point p; p.x = x * v, p.y = y * v; return p; }
    point operator / (int v) const	{ point p; p.x = x / v, p.y = y / v; return p; }
    bool  operator == (point p) const	{ return bool(x == p.x && y == p.y); }
    bool  operator != (point p) const	{ return bool(x != p.x || y != p.y); }
    bool  contains(int v) const	{ return bool(v >= x && v < y); }
}; /* point */


/*---------------------------------------------------------*/
struct rect{
    point a, b;

    rect() {}
    rect(int ax, int ay, int bx, int by) :a(ax, ay), b(bx, by) {}
    rect(point a, point b) :a(a), b(b) {}

    void  move(int dX, int dY) { a.x += dX; a.y += dY; b.x += dX; b.y += dY; }
    void  grow(int dX, int dY) { a.x -= dX; a.y -= dY; b.x += dX; b.y += dY; }

    rect& FAST_ operator *= (rect);
    rect& FAST_ operator += (rect);
    rect  FAST_ operator * (rect) const;
    rect  FAST_ operator + (rect) const;

    bool operator == (rect r) const { return bool(a == r.a && b == r.b); }
    bool operator != (rect r) const { return bool(a != r.a || b != r.b); }

    bool isEmpty() const 	 { return bool(a.x >= b.x || a.y >= b.y); }
    bool contains(int x, int y) const { return bool(x >= a.x && x < b.x
						 && y >= a.y && y < b.y); }
    bool contains(point p) const { return contains(p.x, p.y); }
}; /* rect */

#endif /* _h_POINT_ */
